@@ -16,9 +16,12 @@
 	2014.02.04 V00.10 Bug fixed CallCheck
 	2014.02.04 V00.11 Added  ReloadCallCheckACL
 	2014.02.05 V00.12 Added log file
+	2014.02.06 V00.13 Bug fixed
+	2014.02.06 V00.14 Added KeepAlive for TCP
 */
 
 #include "dsgwd_pi.h"
+#include "version.h"
 #include "node.h"
 
 void	exit (int status);
@@ -59,7 +62,8 @@ char	RelaySW;
 char	respFlags;
 char	Connected;
 char	DVwait;
-unsigned char	PacketIdSave[2];
+unsigned char	SndPacketIdSave[2];
+unsigned char	RcvPacketIdSave[2];
 
 char	PTT_On;
 char	COS_On;
@@ -83,7 +87,7 @@ void CloseSocket()
 
         logfile = fopen (LOGFILE, "a");
 	now = time(NULL);
-        fprintf (logfile, "%24.24s dsgwd_pi %s end\n", ctime(&now), VERSION);
+        fprintf (logfile, "%24.24s dsgwd_pi %s End\n", ctime(&now), VERSION);
         fclose(logfile);
         exit(0);
 }
@@ -127,11 +131,14 @@ void	send_dsvt(void)
 		{
 			if (send_pkt.B_header.seq == 0x80)
 			{
+        			SndPacketIdSave[0] = send_pkt.B_header.PacketID[0];
+        			SndPacketIdSave[1] = send_pkt.B_header.PacketID[1];
 				ConSend();
 				ConnectWait = TRUE;
 				AckTimeOut = 0;
 				DSTREQcnt = 0;
 				Connected = FALSE;
+				RelaySW = FALSE;
 				return;
 			}
 			if (RelaySW)
@@ -160,7 +167,6 @@ void	send_dsvt(void)
 			{
 				ConnectWait = FALSE;
 				FirstHeaderSend = FALSE;
-				RelaySW = FALSE;
 				Connected = FALSE;
 			}
 		}
@@ -202,16 +208,15 @@ void	receive_dsvt(void)
 			Connected = TRUE;
 		}
         }
-
 	if (!memcmp ((char *)&inet_recv.ID, STREAM_ID, 4) 
-		&& !memcmp(&PacketIdSave, &inet_recv.B_header.PacketID, 2))
+		&& !memcmp(&RcvPacketIdSave, &inet_recv.B_header.PacketID, 2))
 	{
 		mq_send (inet_qid, (char *)&inet_recv, sizeof(inet_recv), 1);
 		DVwait = FALSE;
 	}
 }
 
-void	AckSendTrust(char srcRptCall[8], char PacketID[2] )
+void	AckSendTrust(char srcRptCall[], char PacketID[] )
 {
         struct  shm_form        ack_msg;
 
@@ -228,16 +233,19 @@ void	AckSendTrust(char srcRptCall[8], char PacketID[2] )
 	return;
 }
 
-void    NakSendTrust(char srcRptCall[8])
+void    NakSendTrust(char srcRptCall[], char PacketID[])
 {
         struct  shm_form        ack_msg;
 
         ack_msg.type = Nak;
         memcpy (ack_msg.Rep1Call, srcRptCall, 8);
         memcpy (ack_msg.dstCall, client_call, 8);
+	ack_msg.PacketID[0] = PacketID[0];
+	ack_msg.PacketID[1] = PacketID[1];
         sndAddr.sin_family = PF_INET;
         sndAddr.sin_port = trust_db_port;
         sndAddr.sin_addr = dbIP;
+	
         sendto (udpSocket, (char *)&ack_msg, sizeof(ack_msg), 0,
                  (struct sockaddr *)&sndAddr, sizeof(sndAddr));
         return;
@@ -258,12 +266,8 @@ void	ConSend(void)
 	memcpy (&send_msg.dstRep, &send_pkt.header.YourCall, 8);
 	send_msg.type = ReqConnect;
 	send_msg.protocol = G3;
-	send_msg.PacketID[0] = send_pkt.B_header.PacketID[0];
-        send_msg.PacketID[1] = send_pkt.B_header.PacketID[1];
-	PacketIdSave[0] = send_pkt.B_header.PacketID[0];
-	PacketIdSave[1] = send_pkt.B_header.PacketID[1];
-	send_msg.PacketID[0] = PacketIdSave[0];
-	send_msg.PacketID[1] = PacketIdSave[1];
+	send_msg.PacketID[0] = SndPacketIdSave[0];
+	send_msg.PacketID[1] = SndPacketIdSave[1];
 	sndAddr.sin_family = PF_INET;
 	sndAddr.sin_port = trust_db_port;
 	sndAddr.sin_addr = dbIP;
@@ -290,6 +294,7 @@ int main(int argc, const char **argv)
 
 	FILE	*PidFile;
 	int	cmdin;
+	int	KeepAliveCnt;
 
         signal(SIGINT,CloseSocket);
         signal(SIGTERM,CloseSocket);
@@ -297,7 +302,7 @@ int main(int argc, const char **argv)
 
 	logfile = fopen (LOGFILE, "a");
 	now = time(NULL);
-	fprintf (logfile, "%24.24s dsgwd_pi %s start\n", ctime(&now), VERSION);
+	fprintf (logfile, "%24.24s dsgwd_pi %s Start\n", ctime(&now), VERSION);
 	fclose(logfile);
 
 	srand ((unsigned) time (NULL));
@@ -329,6 +334,7 @@ int main(int argc, const char **argv)
 	Connected = FALSE;
 	DSTREQcnt = 0;
 	DVwait = FALSE;
+	KeepAliveCnt = 0;
 	NodeVendorID =  VendorID;
 	NodeProductID = ProductID;
 
@@ -341,7 +347,10 @@ int main(int argc, const char **argv)
 	else
 	{
 		/* コマンド入力の名前付きのパイプ(FIFO)のオープン */
-		cmdin = open ("/tmp/dsgwd_pi-cmdin", O_NONBLOCK | O_RDONLY);
+
+		mkfifo ("/tmp/dsgwd_pi-cmdin", 0600);
+
+		cmdin = open ("/tmp/dsgwd_pi-cmdin", O_RDONLY | O_NONBLOCK);
 		if (cmdin < 0)
 		{
         		logfile = fopen (LOGFILE, "a");
@@ -381,7 +390,8 @@ int main(int argc, const char **argv)
 	{
         	logfile = fopen (LOGFILE, "a");
         	now = time(NULL);
-		fprintf (logfile, "%24.24s pthread error %s\n", ctime(&now), strerror(status));
+		fprintf (logfile, "%24.24s pthread error %s\n", 
+			ctime(&now), strerror(status));
 		fclose (logfile);
 		exit(-1);
 	}
@@ -401,13 +411,16 @@ int main(int argc, const char **argv)
 			sizeof(dstAddr)) < 0){
         	logfile = fopen (LOGFILE, "a");
         	now = time(NULL);
-		fprintf(logfile, "%24.24s Does not connect to '%s'.\n",ctime(&now),TrustDomainName);
+		fprintf(logfile, "%24.24s Does not connect to '%s'.\n",
+			ctime(&now),TrustDomainName);
 		fclose (logfile);
 		CloseSocket();	
 	}
+
         logfile = fopen (LOGFILE, "a");
         now = time(NULL);
-	fprintf(logfile, "%24.24s Connect to '%s'.\n", ctime(&now), TrustDomainName);
+	fprintf(logfile, "%24.24s Connect to '%s'.\n", 
+		ctime(&now), TrustDomainName);
 	fclose (logfile);
 
 	//コールサイン入力のプロンプト
@@ -423,7 +436,6 @@ int main(int argc, const char **argv)
 		client_call[n] = toupper (client_call[n]);
 	}
 	send (dstSocket, client_call, 8, 0);
-
         //パスワード入力のプロンプト
 	memset (buf, 0x20, 9);
 	while (memcmp (buf, "Password:", 9) != 0)
@@ -433,7 +445,6 @@ int main(int argc, const char **argv)
 
         //パスワードの送信
         send (dstSocket, client_password, strlen(client_password), 0);
-
         //ポート番号入力のプロンプト
 	memset (buf, 0x20, 12);
 	numrcv = recv (dstSocket, buf, 128, 0);
@@ -441,7 +452,8 @@ int main(int argc, const char **argv)
 	{
         	logfile = fopen (LOGFILE, "a");
         	now = time(NULL);
-		fprintf (logfile, "%24.24s Invalid CallSign or Password\n", ctime(&now));
+		fprintf (logfile, "%24.24s Invalid CallSign or Password\n", 
+			ctime(&now));
 		fclose (logfile);
 		CloseSocket();
 	}
@@ -507,6 +519,14 @@ int main(int argc, const char **argv)
 						ReqSendSender();
 					}
 				}
+				KeepAliveCnt++;
+				if (KeepAliveCnt > 2000)
+				{
+                                        reqConMsg.reqType = KeepAlive;
+					memset (&reqConMsg.Keep_Alive, 0x00, 3);
+                                        send (dstSocket, (char *)&reqConMsg, 4, 0);
+					KeepAliveCnt = 0;
+				}
 				break;
 			default:
                                 if (FD_ISSET(udpSocket, &readOk))
@@ -526,7 +546,7 @@ int main(int argc, const char **argv)
                                         }
                         		if (reqConMsg.reqType == Ack)
                         		{
-						if (!Connected && !memcmp(&PacketIdSave, &reqConMsg.ackConnect.PacketID, 2))
+						if (!Connected && !memcmp(&SndPacketIdSave, &reqConMsg.ackConnect.PacketID, 2))
 						{
 							dstPort = reqConMsg.ackConnect.port;
 							dstIP = reqConMsg.ackConnect.ip;
@@ -560,18 +580,18 @@ int main(int argc, const char **argv)
 							inet_ntoa(reqConMsg.reqConnect.ip),
 							ntohs(reqConMsg.reqConnect.port));
 						fclose (logfile);
+                                                RcvPacketIdSave[0] = reqConMsg.reqConnect.PacketID[0];
+                                                RcvPacketIdSave[1] = reqConMsg.reqConnect.PacketID[1];						
 						if (PTT_On || COS_On)
 						{
-							NakSendTrust(reqConMsg.reqConnect.Rpt1Call);
+							NakSendTrust(reqConMsg.reqConnect.Rpt1Call, RcvPacketIdSave);
 						}
 						else
 						{
-							PacketIdSave[0] = reqConMsg.reqConnect.PacketID[0];
-							PacketIdSave[1] = reqConMsg.reqConnect.PacketID[1];
 							ReqSendSender();
 							DVwait = TRUE;
 							DSTREQcnt = 0;
-							AckSendTrust (reqConMsg.reqConnect.Rpt1Call, reqConMsg.reqConnect.PacketID);
+							AckSendTrust (reqConMsg.reqConnect.Rpt1Call, RcvPacketIdSave);
 						}
                         		}
 				}
@@ -579,7 +599,7 @@ int main(int argc, const char **argv)
 				{
 					send_dsvt();
 				}
-				if(FD_ISSET(cmdin, &readOk))
+				if (FD_ISSET(cmdin, &readOk))
 				{
                 			read (cmdin, buf, 64);
                 			if (!memcmp (buf, "end", 3))
