@@ -3,7 +3,6 @@
 	Date		2014.01.20
 */
 
-#define	VERSION	"V00.12"
 #define	LOGFILE	"/var/log/dsgwd_pi.log"
 
 #include <stdio.h>
@@ -20,6 +19,7 @@
 #include <mqueue.h>
 #include <pthread.h>
 #include <errno.h>
+#include <time.h>
 
 
 #define FALSE           0
@@ -81,6 +81,7 @@ void	ConSend(void);
 #define		Ack		0x02
 #define		Nak		0x03		/* Busy	*/
 #define		NoRoute		0x04
+#define		KeepAlive	0x05
 #define		ReqEnd		0xff
 
 
@@ -108,6 +109,8 @@ struct	repMsg
 							/* 0x01: ReqConnect */
 							/* 0x02: Ack	    */
 							/* 0x03: Nak	    */
+							/* 0x04: NoRoute    */
+							/* 0x05: KeepAlive  */
 							/* 0xff: ReqEnd     */
 	union {
 		struct
@@ -142,6 +145,10 @@ struct	repMsg
 			unsigned char	PacketID[2];
 			unsigned char	RespFlags;
 		} ackConnect;
+		struct
+		{
+			char		reserve[3];
+		} Keep_Alive;
 	};
 };
 
